package com.huiju.welcome.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaGoods;
import com.huiju.welcome.model.TaGoodsImg;
import com.huiju.welcome.model.TaGoodsSpec;
import com.huiju.welcome.model.TdGoodsType;
import com.huiju.welcome.service.ITaGoodsImgService;
import com.huiju.welcome.service.ITaGoodsService;
import com.huiju.welcome.service.ITaGoodsSpecService;
import com.huiju.welcome.utils.StatusUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;


/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-07-16
 */
@RestController
@RequestMapping("/")
public class TaGoodsController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaGoodsController.class);

    @Autowired
    public ITaGoodsService iTaGoodsService;
    @Autowired
    public ITaGoodsImgService iTaGoodsImgService;
    @Autowired
    public ITaGoodsSpecService iTaGoodsSpecService;


    /**
     * 保存对象
     * @param paramStr 实体对象
     * @return
     */
    @RequestMapping(value="/taGoods/add",method= RequestMethod.POST)
    public ResponseBean tpShopImgAdd(@RequestBody String paramStr){
        JSONObject jsonObject = JSONObject.parseObject(paramStr);

        TaGoods taGoods = jsonObject.getObject("goods", TaGoods.class);
        TaGoodsImg taGoodsImg = jsonObject.getObject("img",TaGoodsImg.class);
        JSONArray spec = jsonObject.getJSONArray("spec");

        ResponseBean responseBean = new ResponseBean();
        try {
            taGoods.setCreateDate(LocalDateTime.now());
            taGoods.setStatus(StatusUtils.Normal);
            Boolean flag = iTaGoodsService.save(taGoods);
            Boolean flagImg = iTaGoodsImgService.save(taGoodsImg);
            List<TaGoodsSpec> specList = spec.toJavaList(TaGoodsSpec.class);
            specList.stream().forEach(e -> {
                e.setGoodsId(taGoods.getGoodsId());
            });

            boolean result = iTaGoodsSpecService.saveBatch(specList);

            if (flag&&flagImg&&result){
                responseBean.addSuccess(taGoods);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taGoodsAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taGoods/delete/{id}", method= RequestMethod.DELETE)
    public ResponseBean taGoodsDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            TaGoods taGoods = iTaGoodsService.getById(id);
            taGoods.setStatus(StatusUtils.Delete);
            if(iTaGoodsService.updateById(taGoods)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taGoodsDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param paramStr 实体对象
     * @return
     */
    @RequestMapping(value="/taGoods/update",method= RequestMethod.PUT)
    public ResponseBean tpShopImgUpdate(@RequestBody String paramStr){
        ResponseBean responseBean = new ResponseBean();
        JSONObject jsonObject = JSONObject.parseObject(paramStr);
        TaGoods taGoods = jsonObject.getObject("goods", TaGoods.class);
        TaGoodsImg taGoodsImg = jsonObject.getObject("img",TaGoodsImg.class);
        JSONArray spec = jsonObject.getJSONArray("spec");

        try {
            Boolean flag = iTaGoodsService.updateById(taGoods);
            Boolean flagImg = iTaGoodsImgService.updateById(taGoodsImg);
            Boolean flagSpec = false;
            for(int i = 0;i<spec.size();i++){
                TaGoodsSpec taGoodsSpec = spec.getObject(i, TaGoodsSpec.class);
                flagSpec = iTaGoodsSpecService.updateById(taGoodsSpec);
            }
            if (flag&& flagImg&&flagSpec){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            logger.error("taGoodsUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taGoods/get/{id}",method= RequestMethod.GET)
    public ResponseBean tpShopImgGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            TaGoods taGoods = iTaGoodsService.getById(id);
            taGoods.setSpecList(iTaGoodsSpecService.specListByGoodsId(id));
            responseBean.addSuccess(taGoods);
        }catch (Exception e){
            logger.error("taGoodsDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    @ApiOperation(value = "商品列表", notes = "商品列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "typeName", value = "商品名称")
    })
    @RequestMapping(value = "/taGoods", method = RequestMethod.GET)
    public ResponseBean typeList(@RequestParam(defaultValue = "1") int pageNum,
                                 @RequestParam(defaultValue = "10") int pageSize,
                                 @RequestParam(value = "goodsName",required = false) String goodsName,
                                 @RequestParam(value = "typeId",required = false) Integer typeId) {
        IPage<TaGoods> pg = new Page<>(pageNum, pageSize);

        ResponseBean userManagement= iTaGoodsService.goodsList(pg,goodsName,typeId);

        return userManagement;
    }






}
