package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.model.TaDevice;
import com.huiju.welcome.model.TaPerson;
import com.huiju.welcome.model.TaSnapshotPerson;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.*;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.HttpURLConnection;
import java.net.URL;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class CameraServiceImpl implements ICameraService {
    @Autowired
    private ITaDeviceService taDeviceService;

    @Autowired
    TencentFaceUtils tencentFaceUtils;

    @Autowired
    AliFaceUtils aliFaceUtils;

    @Autowired
    ISysParamService iSysParamService;

    @Autowired
    ITaPersonService taPersonService;

    @Autowired
    ITaSnapshotPersonService taSnapshotPersonService;

    @Autowired
    ITaVisitingLogService visitingLogService;

    @Autowired
    ITaCustomerService iTaCustomerService;

    @Override
    public String dispatchPerson(TaPerson taPerson, TaDevice... devices) {
        List<TaDevice> taDeviceList = null;
        if (null == devices || devices.length == 0) {
            QueryWrapper<TaDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("status",StatusUtils.Normal);
            queryWrapper.eq("afr",StatusUtils.BoolTrue);
            taDeviceList = taDeviceService.list(queryWrapper);
        } else {
            taDeviceList = Arrays.asList(devices);
        }

        //分发人脸到各个相机
        for(TaDevice device: taDeviceList) {
            try {
                Map<String, Object> payload = new HashMap<>();
                payload.put("personId", taPerson.getPersonId());
                payload.put("image", taPerson.getAvatar());

                Action act = Action.create(Action.TYPE_DISPATCH_FACE, payload);

                WebSocketServer.sendInfo(act.toJSON(), "device", device.getDeviceId());
            } catch (Exception e){
                e.printStackTrace();
                return "连接设备(" + device.getDeviceId() + ")人脸下发服务 失败: "+ e.getMessage();
            }
        }
//
//        // 下发到腾讯云人脸识别
//        try {
//            tencentFaceUtils.createFace(String.valueOf(taPerson.getRealId()), taPerson.getAvatar());
//        } catch (TencentCloudSDKException e) {
//            e.printStackTrace();
//            log.error("下发人脸到腾讯云失败: " + e.getMessage());
//        }

        try {
            aliFaceUtils.createFace(String.valueOf(taPerson.getRealId()), taPerson.getAvatar());
        } catch (Exception e) {
            e.printStackTrace();
            log.error("下发人脸到阿里云失败: " + e.getMessage());
        }

        return "";
    }

    @Override
    public boolean newPersonByDevice(TaSnapshotPerson taSnapshotPerson) {
        boolean workingTime = iSysParamService.isInWorkingTime(LocalDateTime.now());
        if (!workingTime) {
            taSnapshotPersonService.save(taSnapshotPerson);
            return true;
        }

        // 人像质量最低值, 低于这个值的认为人脸抓拍失败, 取值范围 1 - 100
        Float faceMinScore = iSysParamService.getFaceMinScore();
        Integer score = taSnapshotPerson.getScore();
        if (null == score || score.intValue() < faceMinScore) {
            log.error("人脸质量过低, 放弃本次抓拍人脸 {}", taSnapshotPerson.getAvatar());
            return false;
        }

        // 人脸相似度阈值, 低于设定认为没有识别成功
        // SDK 中这个值范围是 0.001 - 1, 本系统设定范围是 1 - 100
        Float samePersonThreshold = iSysParamService.getSamePersonThreshold();
        Float similarity = taSnapshotPerson.getSimilarity();

        // 识别结果
        boolean newGuy = null == similarity || similarity < (samePersonThreshold / 100);
        TaPerson taPerson = null;
        Integer personIdOrigin = null;

        if (!newGuy) {
            if (null != taSnapshotPerson.getMatchPerson()) {
                taPerson = taPersonService.getById(taSnapshotPerson.getMatchPerson());
                if (null == taPerson) {
                    // 出现这种情况, 说明阿里云的人脸库与本地不一致。
                    newGuy = true;

                    // 保留原来人员ID
                    personIdOrigin = taSnapshotPerson.getMatchPerson();
                }
            }

            // 如果是工作人员则忽略
            if (null != taPerson && null != taPerson.getTypeId() && taPerson.getTypeId() == CommConstant.TYPE_INNER_PERSON) {
                taSnapshotPerson.setMatchPerson(taPerson.getRealId());
                taSnapshotPersonService.save(taSnapshotPerson);
                return true;
            }
        }

        if (!newGuy) {
            // 抓拍人与比对人是同一个人
            taSnapshotPerson.setPersonId(taSnapshotPerson.getMatchPerson());

            LocalDateTime lastTime = taPerson.getLastVisiteDate();
            boolean sameDevice = taSnapshotPerson.getDeviceId().equals(taPerson.getLastVisiteDevice());

            if (null != lastTime && sameDevice) {
                // 降噪设定
                Float dispatchTimeLimit = iSysParamService.getDispatchTimeLimit();

                // 默认 2 分钟内连续识别的都是垃圾数据
                Duration duration = Duration.between(lastTime, taSnapshotPerson.getCreateDate());
                if (duration.getSeconds() < (60 * dispatchTimeLimit)) {
                    return true;
                }
            }
        }

        if (newGuy) {
            // 新人入库, 同时写入抓拍人ID
            if (null != personIdOrigin) {
                taSnapshotPerson.setMatchPerson(personIdOrigin);
            }

            taPerson = taPersonService.newPersonByDevice(taSnapshotPerson);

            // 同步相机识别库
            dispatchPerson(taPerson);
        }

        // 推送消息
        taPersonService.send2Front(taPerson);

        // 保存抓拍日志
        taSnapshotPersonService.save(taSnapshotPerson);

        // 预写到访日志
        visitingLogService.newLogByDevice(taSnapshotPerson, taPerson);

        // 判断是否客户来访
        try {
            iTaCustomerService.judgeNewCust(taPerson);
        } catch (Exception e) {
            log.error(e.getMessage());
        }

        return true;
    }
}
